// - ADT-
// (c)   2002

#include "AdtFile.h"
#include <windows.h>

#define ADT_HEAD_SIZE 256
#define ADT_SIGN_SIZE 22
#define ADT_NAME_SIZE 16
#define ADT_TAIL_SIZE 210

#pragma pack(push, 1)
struct AdtHeader			// adt-                              
{                                                                                          
	char	sign[ADT_SIGN_SIZE];	// (.. "CASIO ADDIN DATAFILE"+0x1A+0x00)
	DWORD	filesize;		//  (4 )                            
	DWORD	recCount;		//-                                    
	char	name[ADT_NAME_SIZE];	// adt-                                    
	char	tail[ADT_TAIL_SIZE];	//""                                 
};
#pragma pack(pop)

const char* adtSign="CASIO ADDIN DATAFILE\x1A";

unsigned char AdtFile::mask[20]=
	{0x83,0x4A,0x83,0x56,0x83,0x49,0x8C,0x76,0x8E,0x5A,
	 0x8B,0x40,0x8A,0x94,0x8E,0xAE,0x89,0xEF,0x8E,0xD0};


AdtFile::AdtFile()
{
	size=0;
	point=NULL;
}

AdtFile::~AdtFile()
{
	delAll();
}

int AdtFile::loadFile(const char* fname)
{
	HANDLE file;
	DWORD readCount,recSize;
	AdtHeader header;
	unsigned char *buffer;
	int i;

	if (!checkFile(fname))
		return 0;

	if (point!=NULL)
		delAll();

	// 
	file=CreateFile(fname,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);

	// 
	ReadFile(file,&header,ADT_HEAD_SIZE,&readCount,NULL);

	size=header.recCount;
	point=new AdtRecord*[header.recCount];

	for(i=0; i<size; i++)
	{
		ReadFile(file,&recSize,sizeof(recSize),&readCount,NULL);
		buffer=new unsigned char[recSize];
		ReadFile(file,buffer,recSize,&readCount,NULL);
		code(buffer,recSize);
		point[i]=new AdtRecord(recSize,buffer);
		delete [] buffer;
	}

	CloseHandle(file);
	return 1;
}

int AdtFile::saveFile(const char* fname, const char* adtName)
{
	HANDLE file;
	AdtHeader header;
	DWORD totalSize=0,writeCount,recSize;
	unsigned char *buffer;
	int i;

	file=CreateFile(fname,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,0,NULL);
	if (file==INVALID_HANDLE_VALUE)
	{
		return 0;
	}
	
	//  
	strcpy(header.sign,adtSign);
	for(i=0; i<size; i++)
		totalSize+=point[i]->getSize();
	header.filesize=totalSize+ADT_HEAD_SIZE+size*4;
	header.recCount=size;
	strcpy(header.name,adtName);
	for(i=strlen(adtName); i<ADT_NAME_SIZE; i++)
		header.name[i]=0;

	for(i=0; i<ADT_TAIL_SIZE; i++)
		header.tail[i]=0;

	// 
	WriteFile(file,&header,ADT_HEAD_SIZE,&writeCount,NULL);

	//  
	for(i=0; i<size; i++)
	{
		recSize=point[i]->getSize();
		WriteFile(file,&recSize,sizeof(recSize),&writeCount,NULL);
		buffer=point[i]->getData();
		code(buffer,recSize);
		WriteFile(file,buffer,recSize,&writeCount,NULL);
		delete [] buffer;
	}
	CloseHandle(file);
	return 1;
}

int AdtFile::checkFile(const char* fname)
{
	HANDLE file;
	DWORD fsize,readCount,recSize;
	AdtHeader header;
	int result,i;
	char *buffer;

	// 
	file=CreateFile(fname,GENERIC_READ,0,NULL,OPEN_EXISTING,0,NULL);
	if (file==INVALID_HANDLE_VALUE)
		return 0;

	//  
	fsize=GetFileSize(file,NULL);
	if (fsize<sizeof(AdtHeader))
	{
		CloseHandle(file);
		return 0;
	}
	
	//SetFilePointer(file,0,0,FILE_BEGIN);
	// 
	ReadFile(file,&header,ADT_HEAD_SIZE,&readCount,NULL);
	if (strcmp(header.sign,adtSign)!=0)
	{
		CloseHandle(file);
		return 0;
	}

	if (header.filesize!=fsize)
	{
		CloseHandle(file);
		return 0;
	}

	for(i=0; i<(int)header.recCount; i++)
	{
		result=ReadFile(file,&recSize,sizeof(recSize),&readCount,NULL);
		if (result&&readCount==0)
		{
			CloseHandle(file);
			return 0;
		}
		if (recSize)
		{
			buffer=new char[recSize];
			result=ReadFile(file,buffer,recSize,&readCount,NULL);
			delete [] buffer;
			if (result&&readCount==0)
			{
				CloseHandle(file);
				return 0;
			}
		}
	}

	CloseHandle(file);
	return 1;
}

int AdtFile::add(AdtRecord* rec)
{
	AdtRecord **point2;
	int i;

	point2=new AdtRecord*[size+1];
	for(i=0; i<size; i++)
		point2[i]=point[i];
	point2[size]=rec;

	if (size>0)
		delete [] point;

	point=point2;
	size++;

	return size;
}

int AdtFile::del(int num)
{
	AdtRecord **point2;
	int i;

	if (num<size)
	{
		delete point[num];
		size--;
		if (size==0)
		{
			point=NULL;
		}
		else
		{
			point2=new AdtRecord*[size];
			for(i=0; i<num; i++)
				point2[i]=point[i];
			for(i=num; i<size; i++)
				point2[i]=point[i+1];
			
			delete [] point;
			point=point2;
		}
	}

	return size;
}

int AdtFile::delAll()
{
	if (point!=NULL)
	{
		for(int i=0; i<size; i++)
			delete point[i];
		delete [] point;
		size=0;
		point=NULL;
	}

	return size;
}


void AdtFile::code(unsigned char* seq, int slen)
{
	int i,j;

	for(i=0, j=0; i<slen; i++,j==19?j=0:j++)
		seq[i]^=mask[j];
}

void AdtFile::swap(int num1, int num2)
{
	AdtRecord *tmp;

	if (num1>=0&&num2>=0&&num1<size&&num2<size)
	{
		tmp=point[num1];
		point[num1]=point[num2];
		point[num2]=tmp;
	}

}
